@@ -12,7 +12,7 @@ use File::Path ();
 use File::Basename ();
 
 use vars qw($VERSION @ISA);
-$VERSION = '0.19_05';
+$VERSION = '0.20_01';
 
 # Okay, this is the brute-force method of finding out what kind of
 # platform we're on.  I don't know of a systematic way.  These values
@@ -570,6 +570,15 @@ generated PPD.
 
 =back
 
+=item subclass()
+
+This creates a new C<Module::Build> subclass on the fly, as described
+in the L<SUBCLASSING> section.  The caller must provide either a
+C<class> or C<code> parameter, or both.  The C<class> parameter
+indicates the name to use for the new subclass, and defaults to
+C<MyModuleBuilder>.  The C<code> parameter specifies Perl code to use
+as the body of the subclass.
+
 =item create_build_script()
 
 Creates an executable script called C<Build> in the current directory
@@ -1392,6 +1401,8 @@ Behind the scenes, this actually does create a C<.pm> file, since the
 code you provide must persist after Build.PL is run if it is to be
 very useful.
 
+See also the documentation for the C<subclass()> method.
+
 
 =head1 MOTIVATIONS
 
