@@ -14,8 +14,46 @@ use IO::File ();
 
 #################### Constructors ###########################
 sub new {
-  my $package = shift;
-  my %input = @_;
+  my $self = shift()->_construct(@_);
+
+  $self->cull_args(@ARGV);
+  
+  die "Too early to specify a build action '$self->{action}'.  Do 'Build $self->{action}' instead.\n"
+    if $self->{action};
+
+  $self->_set_install_paths;
+  $self->dist_name;
+  $self->check_manifest;
+  $self->check_prereq;
+  $self->dist_version;
+
+  return $self;
+}
+
+sub resume {
+  my $self = shift()->_construct(@_);
+  
+  $self->read_config;
+  
+  my $perl = $self->find_perl_interpreter;
+  warn(" * WARNING: Configuration was initially created with '$self->{properties}{perl}',\n".
+       "   but we are now using '$perl'.\n")
+    unless $perl eq $self->{properties}{perl};
+  
+  $self->cull_args(@ARGV);
+  $self->{action} ||= 'build';
+  
+  return $self;
+}
+
+sub current {
+  # hmm, wonder what the right thing to do here is
+  local @ARGV;
+  return shift()->resume;
+}
+
+sub _construct {
+  my ($package, %input) = @_;
 
   my $args   = delete $input{args}   || {};
   my $config = delete $input{config} || {};
@@ -48,7 +86,7 @@ sub new {
 				  },
 		   }, $package;
 
-  my $p = $self->{properties};
+  my ($p, $c) = ($self->{properties}, $self->{config});
   $p->{bindoc_dirs} ||= [ "$p->{blib}/script" ];
   $p->{libdoc_dirs} ||= [ "$p->{blib}/lib", "$p->{blib}/arch" ];
 
@@ -56,51 +94,18 @@ sub new {
   $p->{requires} = delete $p->{prereq} if exists $p->{prereq};
   $p->{script_files} = delete $p->{scripts} if exists $p->{scripts};
 
-  $self->cull_args(@ARGV);
-  
-  die "Too early to specify a build action '$self->{action}'.  Do 'Build $self->{action}' instead.\n"
-    if $self->{action};
-
   $self->add_to_cleanup( @{delete $p->{add_to_cleanup}} )
     if $p->{add_to_cleanup};
-  
-  $self->_set_install_paths;
-  $self->dist_name;
-  $self->check_manifest;
-  $self->check_prereq;
-  $self->dist_version;
 
-  return $self;
-}
-
-sub resume {
-  my $package = shift;
-  my $self = bless {@_}, $package;
-  
-  $self->read_config;
-  
-  my $perl = $self->find_perl_interpreter;
-  warn(" * WARNING: Configuration was initially created with '$self->{properties}{perl}',\n".
-       "   but we are now using '$perl'.\n")
-    unless $perl eq $self->{properties}{perl};
-  
-  $self->cull_args(@ARGV);
-  $self->{action} ||= 'build';
+  # perl 5.8.1-RC[1-3] had some broken %Config entries, and
+  # unfortunately Red Hat 9 shipped it like that.  Fix 'em up here.
+  for (qw(siteman1 siteman3 vendorman1 vendorman3)) {
+    $c->{"install${_}dir"} ||= $c->{"install${_}"};
+  }
   
   return $self;
 }
 
-sub current {
-  my $package = shift;
-  # hmm, wonder what the right thing to do here is
-  local @ARGV;
-  return $package->resume(
-			  properties => {
-					 config_dir => '_build',
-					},
-			 );
-}
-
 ################## End constructors #########################
 
 sub _set_install_paths {
@@ -441,11 +446,8 @@ sub _persistent_hash_write {
   if (my $file = $self->config_file($name)) {
     return if -e $file and !keys %{ $ph->{new} };  # Nothing to do
     
-    local $Data::Dumper::Terse = 1;
-    my $fh = IO::File->new("> $file") or die "Can't write to $file: $!";
     @{$ph->{disk}}{ keys %{$ph->{new}} } = values %{$ph->{new}};  # Merge
-    print $fh Data::Dumper::Dumper($ph->{disk});
-    close $fh;
+    $self->_write_dumper($name, $ph->{disk});
     
     $ph->{new} = {};
   }
@@ -496,8 +498,8 @@ sub cleanup {
 
 sub config_file {
   my $self = shift;
-  return unless -d $self->{properties}{config_dir};
-  return File::Spec->catfile($self->{properties}{config_dir}, @_);
+  return unless -d $self->config_dir;
+  return File::Spec->catfile($self->config_dir, @_);
 }
 
 sub read_config {
@@ -516,24 +518,24 @@ sub read_config {
   }
 }
 
+sub _write_dumper {
+  my ($self, $filename, $data) = @_;
+  
+  my $file = $self->config_file($filename);
+  my $fh = IO::File->new("> $file") or die "Can't create '$file': $!";
+  local $Data::Dumper::Terse = 1;
+  print $fh Data::Dumper::Dumper($data);
+}
+
 sub write_config {
   my ($self) = @_;
   
   File::Path::mkpath($self->{properties}{config_dir});
   -d $self->{properties}{config_dir} or die "Can't mkdir $self->{properties}{config_dir}: $!";
   
-  local $Data::Dumper::Terse = 1;
-
-  my $file = $self->config_file('build_params');
-  my $fh = IO::File->new("> $file") or die "Can't create '$file': $!";
-  print $fh Data::Dumper::Dumper([$self->{args}, $self->{config}, $self->{properties}]);
-  close $fh;
-
-  $file = $self->config_file('prereqs');
-  open $fh, "> $file" or die "Can't create '$file': $!";
   my @items = qw(requires build_requires conflicts recommends);
-  print $fh Data::Dumper::Dumper( { map { $_, $self->$_() } @items } );
-  close $fh;
+  $self->_write_dumper('prereqs', { map { $_, $self->$_() } @items });
+  $self->_write_dumper('build_params', [$self->{args}, $self->{config}, $self->{properties}]);
 
   $self->_persistent_hash_write('cleanup');
 }
@@ -991,7 +993,7 @@ sub ACTION_test {
 
   if (@$tests) {
     # Work around a Test::Harness bug that loses the particular perl we're running under
-    local $^X = $self->{config}{perlpath} unless $Test::Harness::VERSION gt '2.01';
+    local $^X = $p->{perl} unless $Test::Harness::VERSION gt '2.01';
     Test::Harness::runtests(@$tests);
   } else {
     print("No tests defined.\n");
@@ -1094,7 +1096,10 @@ sub process_xs_files {
   my $self = shift;
   my $files = $self->find_xs_files;
   while (my ($from, $to) = each %$files) {
-    $self->copy_if_modified( from => $from, to => $to ) unless $from eq $to;
+    unless ($from eq $to) {
+      $self->add_to_cleanup($to);
+      $self->copy_if_modified( from => $from, to => $to );
+    }
     $self->process_xs($to);
   }
 }
@@ -1417,7 +1422,7 @@ sub ACTION_clean {
 sub ACTION_realclean {
   my ($self) = @_;
   $self->depends_on('clean');
-  $self->delete_filetree($self->{properties}{config_dir}, $self->{properties}{build_script});
+  $self->delete_filetree($self->config_dir, $self->build_script);
 }
 
 sub ACTION_ppd {
@@ -1454,14 +1459,27 @@ sub _sign_dir {
     warn "Couldn't load Module::Signature for 'distsign' action:\n $@\n";
     return;
   }
-
+  
+  # Add SIGNATURE to the MANIFEST
+  {
+    my $manifest = File::Spec->catfile($dir, 'MANIFEST');
+    die "Signing a distribution requires a MANIFEST file" unless -e $manifest;
+    my $mode = (stat $manifest)[2];
+    chmod($mode | 0222, $manifest) or die "Can't make $manifest writable: $!";
+    
+    my $fh = IO::File->new(">> $manifest") or die "Can't add SIGNATURE to $manifest: $!";
+    print $fh "\nSIGNATURE    Added here by Module::Build\n";
+    close $fh;
+    chmod($mode, $manifest);
+  }
+  
   # We protect the signing with an eval{} to make sure we get back to
   # the right directory after a signature failure.  Would be nice if
   # Module::Signature took a directory argument.
   
   my $start_dir = $self->cwd;
   chdir $dir or die "Can't chdir() to $dir: $!";
-  eval {Module::Signature::sign()};
+  eval {local $Module::Signature::Quiet = 1; Module::Signature::sign()};
   my @err = $@ ? ($@) : ();
   chdir $start_dir or push @err, "Can't chdir() back to $start_dir: $!";
   die join "\n", @err if @err;
@@ -1507,7 +1525,7 @@ sub ACTION_distdir {
   my $dist_dir = $self->dist_dir;
   $self->delete_filetree($dist_dir);
   $self->add_to_cleanup($dist_dir);
-  ExtUtils::Manifest::manicopy($dist_files, $dist_dir, 'best');
+  ExtUtils::Manifest::manicopy($dist_files, $dist_dir, 'cp');
   warn "*** Did you forget to add $self->{metafile} to the MANIFEST?\n" unless exists $dist_files->{$self->{metafile}};
   
   $self->_sign_dir($dist_dir) if $self->{properties}{sign};
@@ -1595,6 +1613,9 @@ sub ACTION_distmeta {
     };
 
   $node->{generated_by} = "Module::Build version " . Module::Build->VERSION;
+  
+  # If we're in the distdir, the metafile may exist and be non-writable.
+  $self->delete_filetree($self->{metafile});
 
   # YAML API changed after version 0.30
   my $yaml_sub = $YAML::VERSION le '0.30' ? \&YAML::StoreFile : \&YAML::DumpFile;
@@ -1816,8 +1837,9 @@ sub compile_xs {
     
     my $typemap =  $self->find_module_by_name('ExtUtils::typemap', \@INC);
     my $cf = $self->{config};
+    my $perl = $self->{properties}{perl};
     
-    my $command = (qq{$^X "-I$cf->{installarchlib}" "-I$cf->{installprivlib}" "$xsubpp" -noprototypes } .
+    my $command = (qq{$perl "-I$cf->{installarchlib}" "-I$cf->{installprivlib}" "$xsubpp" -noprototypes } .
 		   qq{-typemap "$typemap" "$file"});
     
     print $command;
@@ -1916,9 +1938,21 @@ sub copy_if_modified {
   my %args = @_ > 3 ? @_ : ( from => shift, to_dir => shift, flatten => shift );
   
   my $file = $args{from};
-  my $to_path = $args{to} || File::Spec->catfile( $args{to_dir}, $args{flatten}
-						  ? File::Basename::basename($file)
-						  : $file );
+  unless (defined $file and length $file) {
+    die "No 'from' parameter given to copy_if_modified";
+  }
+  
+  my $to_path;
+  if (defined $args{to} and length $args{to}) {
+    $to_path = $args{to};
+  } elsif (defined $args{to_dir} and length $args{to_dir}) {
+    $to_path = File::Spec->catfile( $args{to_dir}, $args{flatten}
+				    ? File::Basename::basename($file)
+				    : $file );
+  } else {
+    die "No 'to' or 'to_dir' parameter given to copy_if_modified";
+  }
+  
   return if $self->up_to_date($file, $to_path); # Already fresh
   
   # Create parent directories
